#define Uses_TRect
#define Uses_TEvent
#define Uses_TKeys
#define Uses_TView
#define Uses_TView
#define Uses_TDrawBuffer
#define Uses_TPalette


#include <tchessframe.h>
//#include <cl_chess.h>
#include <ctype.h>
//#include <cl_move.h>
#include <chessboard.h>
#include <chesspiece.h>
#include <board.h>


class TChessView : public TView 
{
  Board * field;
  int sx, sy, ax, ay;
  ChessPiece::piece_t piece;
  bool flip;
  
public:
  TChessView( Board *f, TRect& r );
  TPalette& getPalette() const;
  void draw();
  void handleEvent( TEvent& event );
};

class TShowMove : public TView
{
  char * movestr;
public:
  TShowMove( TRect& r );
  void draw();
  //  void handleEvent();
  void setMove( ChessMove& m );
};



TShowMove::TShowMove( TRect& r )
  : TView( r )
{}

void TShowMove::draw()
{
}



TChessView::TChessView(  Board *f, TRect& r )
  : TView( r ), field( f ), sx(-1), flip( 0 )
{
}

void TChessView::handleEvent( TEvent& event )
{
  TView::handleEvent( event );

  if( event.what == evMouseDown )
    {
      if( mouseInView( event.mouse.where ) )
	{
	  TPoint startP = makeLocal( event.mouse.where );
	  setCursor( startP.x, startP.y );
	  if( field->at( startP.x/3, startP.y ).piece != ChessPiece::EMPTY )
	    {
	      sx=startP.x/3;
	      sy=startP.y;
	      TPoint endP=startP;
	      while( mouseEvent( event, evMouseMove ) )
		if( mouseInView( event.mouse.where ) )
		  endP = makeLocal( event.mouse.where );
		
	      if( endP != startP )
		if( flip )
		  field->move( ChessMove( sx, sy, endP.x/3, endP.y ));
		else
		  field->move( ChessMove( sx, Chess::YSIZE-sy-1, 
				   endP.x/3, Chess::YSIZE-endP.y-1 ));
	      sx=-1;
	      draw();
	    }
	  clearEvent( event );
	}
    }
  
  if( event.what == evKeyboard )
    {
      switch( event.keyDown.keyCode )
	{
	case kbHome:
	  setCursor( 0, Chess::YSIZE-2 );
	  break;
	case kbEnd:
	  setCursor( Chess::XSIZE-1, 1 );
	  break;
	case kbUp:
	  setCursor( cursor.x, (cursor.y-1 + Chess::YSIZE) % Chess::YSIZE );
	  break;
	case kbDown:
 	  setCursor( cursor.x, (cursor.y+1) % Chess::YSIZE );
	  break;
	case kbLeft:
	  setCursor( (cursor.x-3+3*Chess::XSIZE) % (3*Chess::XSIZE), cursor.y );
	  break;
	case kbRight:
	  setCursor( (cursor.x+3) % (3*Chess::XSIZE), cursor.y );
	  break;
	case kbEnter:
	  if( sx > 0 )
	    {
	      if( flip )
		field->move( ChessMove( sx, sy, cursor.x/3, cursor.y ));
	      else
		field->move( ChessMove( sx, Chess::YSIZE-sy-1,
				 cursor.x/3, Chess::YSIZE-cursor.y-1 ));
	      sx=-1;
	    }
	  else
	    {
	      sx=cursor.x/3;
	      sy=cursor.y;
	    }
	}
      draw();
      clearEvent( event );
    }
}



void TChessView::draw()
{
  char col[5];
  TDrawBuffer buf;
  for( int i = 0; i < int(sizeof( col )); i++ )
    col[i]=getColor( i+1 );
  
  int flsx, flsy, flex, fley, rr;
  if( flip )
    {
      flsy=flsx=0;
      flex=Chess::XSIZE;
      fley=Chess::YSIZE;
      rr=1;
    }
  else
    {
      flsy=Chess::YSIZE-1;
      flsx=Chess::XSIZE-1;
      flex=fley=-1;
      rr=-1;
    }
  
  int l=0;
  for( int y=flsy; y != fley; y+=rr )
    {
      for( int x=flsx; x != flex; x+=rr )
	{
	  //	  buf.moveChar( 0, ' ', col[0], Chess::XSIZE*3 );
	  int c= (x%2) ^ (y%2);
	  ChessPiece& f = field->at( x, y );
	  char piece=f.toChar();
	  if( f.color == ChessPiece::BLACK ) 
	    {
	      piece=tolower( piece );
	      //	      c+=2;
	    }
	  if( sx == x )
	    if( (flip && sy == y) || (!flip && sy == Chess::YSIZE - y -1 ) )
	    c=4;
	  buf.moveChar( x*3,  ' ', col[c], 3 );
	  buf.moveChar( x*3+1,  piece, 
			col[c], 1 );
	}
      
      writeLine( 0, l++, Chess::XSIZE*3, 1, buf );
    }
  setCursor( 3* (cursor.x / 3) + 1, cursor.y );
  showCursor();
  
}


const char * CHESSPALETTE = "\x06\x07\0x2\0x3\0x4";

TPalette& TChessView::getPalette() const
{
  static TPalette palette = TPalette( CHESSPALETTE, 
				      sizeof( CHESSPALETTE ) - 1 );
  return palette;
}

  
  
TChessFrame::TChessFrame( Board *field )
  : TWindow( TRect( 1, 1, 3*Chess::XSIZE+3, Chess::YSIZE+5), 
	     "ChessBoard", wnNoNumber ),
	     TWindowInit( &TChessFrame::initFrame )
{
  flags &= ~( wfZoom | wfGrow );
  TRect r=getExtent();
  r.grow(-1,-1);
  r.b.y-=2;
  view = new TChessView( field, r );
  view->options |= ofFramed | ofSelectable;
  view->blockCursor();
  insert( view );
  r.a.y=r.b.y+1;
  r.b.y=r.a.y+1;
  move = new TShowMove( r );
  insert( move );
  growMode = 0;
  view->select();
}

void TChessFrame::showTime( bool on )
{
}

	     
