#include "spiegel.h"

#include <chessgicsparse.h>
#include <chessboard.h>

#include <pthread.h>

class FicsParse : public ChessGicsParse
{
  const char * myname;
  const ChessGicsParse * server2;
public:
  FicsParse( const char * name, const ChessGicsParse * serv2, const char * server, int port=5000 )
    : ChessGicsParse( server, port ), myname(name), server2(serv2)
  {}

  void echo(const string  s)
  { cout << myname << ": " << s; }
  
  void evPosition(const string s)
  { 
    ChessBoard b(s, ChessBoard::ICS );
    server2->sendLn(b.gicsSetBoardString());
    server2->sendLn("wname " + b.getPlayerName( ChessPiece::WHITE ));
    server2->sendLn("bname " + b.getPlayerName( ChessPiece::BLACK ));
    cout << "****************************************************" << endl 
	 << b << endl 
	 << "****************************************************" << endl;
      
  }
};

class GicsParse : public ChessGicsParse
{
  const char * myname;
public:
  GicsParse( const char *name, const char * server, int port=5000 )
    : ChessGicsParse( server, port ), myname( name )
  {}
  void echo(const string  s)
  { cout << myname << ": " << s; }
};

GicsParse gics("GICS", "chess.unix-ag.uni-kl.de");

void* gicsThread( void *)
{
  gics.run();
  gics.done();
}


int main()    
{
  gics.init();
  gics.sendLn(gicsHandle); 
  gics.sendLn(gicsPassword);
  gics.sendLn("\nexamine\n");
  pthread_t th;
  pthread_create( &th, 0, gicsThread, 0 );
  FicsParse fics("FICS", &gics, "freechess.org");
  fics.init();
  fics.sendLn(ficsHandle);
  fics.sendLn(ficsPassword);
  fics.startStdinThread();
  fics.sendLn("\nset style 12");
  fics.sendLn("set bell 0");
  fics.sendLn("\nobs 1\n\n");
  fics.run();
  fics.done();
}
  
  
   
    
  
