/* file: ChessLib/chessboardflags.h
   author: Christoph Bauer
   date: 22.1.00
   license: GPL
   
   ChangeLog:
     o 22.1.00 created
     
   Description
     Die Klasse ChessBoardFlags speichert Informationen ber eine 
     Schachstellung:
       - en passat
       - wer kann noch rochieren
*/

#ifndef __CHESSBOARDFLAGS_H__
#define __CHESSBOARDFLAGS_H__

#include <chesspiece.h>

//#define CHESSBOARD_SMALL_FLAGS


class ChessBoardFlags
{
 public:
  enum{ SHORT=1, LONG=2, BOTH=SHORT+LONG };
  
  
#ifndef CHESSBOARD_BIG_FLAGS
  struct flags_t
    {
      unsigned int wcastle  : 2;
      unsigned int bcastle  : 2;
      unsigned int colToMove : 1;
      int enpassat : 5;
  };
#else
  struct flags_t
    {
      unsigned int wcastle;  
      unsigned int bcastle;
      ChessPiece::color_t colToMove;
      int enpassat;
  };
#endif
 private:
  flags_t flags;
  
 public:
  ChessBoardFlags()
    {
      flags.wcastle=flags.bcastle=BOTH;
      flags.enpassat=-1;
    }
  ChessBoardFlags(const ChessBoardFlags& c)
    : flags(c.flags)
    {}
  
  void setEnpassat(int column)
    { flags.enpassat=column; }
  int getEnpassat()
    { return flags.enpassat; }
  void setWCastle(unsigned int enable=BOTH)
    { assert( enable <= BOTH ); flags.wcastle=enable; }
  bool getWCastle()
    { return flags.wcastle; }
  void setBCastle(unsigned int enable=BOTH)
    { assert( enable <= BOTH ); flags.bcastle=enable; }
  bool getBCastle()
    { return flags.bcastle; }
  ChessPiece::color_t getColToMove()
    { return  ChessPiece::color_t(flags.colToMove); }
  void setColToMove(ChessPiece::color_t col)
    { flags.colToMove=col; }
  
  bool getCastle(ChessPiece::color_t col)
    { assert(col != ChessPiece::NOCOLOR); return col == ChessPiece::WHITE ? getWCastle() : getBCastle(); }
  void setCastle(ChessPiece::color_t col, unsigned int enable=BOTH)
    { assert(col != ChessPiece::NOCOLOR); 
    if( col == ChessPiece::WHITE )
      setWCastle(enable);
    else
      setBCastle(enable);
    }

  char getColToMoveChar()
    { return getColToMove() == ChessPiece::WHITE ? 'w' : 'b'; }
      
      
  ChessBoardFlags& operator=(ChessBoardFlags c)
  { flags=c.flags; return *this; }
  bool operator==(ChessBoardFlags& c);
};


#endif
