#include <chesscomprboard.h>
#include <chessboard.h>
#include <algo.h>

const unsigned char huffman[] =
{ 0, 16+8+4+2+1, 1+2+4+8, 1+2+4, 1+2, 1+2+8, 1 };


const int huffbits[] =
{ 1, 6,             6,         5,      5,     5,  3  };

ChessComprBoard::ChessComprBoard(ChessBoard& board)
{
  
  int pos=0;
  compr=new compr_t;
  compr->refcount=0;
#ifdef __CHESS_NEWSIZE__
#error look here! here is an error: the size 21 is not correct
#endif
  compr->board=(char*)malloc(23);
  fill(compr->board,compr->board+23, 0);
  for( ChessBoard::iterator it = board.begin(); it != board.end(); it++ )
    {
      ChessPiece& p=(*it);
      
      if( p.piece == ChessPiece::EMPTY )
	pos++;
      else
	insert(pos, huffman[p.piece]+(p.color << (huffbits[p.piece]-1)), 
	       huffbits[p.piece]); 
    }
#ifdef __CHESS_NEWSIZE__
#error look here! here is an error: the size 5 is not correct
#endif
  int ep=board.getEnpassat();
  if( ep == -1 )
    insert(pos, 1 << 4, 5);
  else
    insert(pos, ep, 5);
  insert(pos, board.getColToMove(), 1);
  insert(pos, board.getWCastle(), 2);
  insert(pos, board.getBCastle(), 2);
  
  // maybe we can save two bytes ;)
  char * reall = (char*) realloc( compr->board, (pos /8) + ((pos % 8) > 0) );
  if( reall != 0 )
    compr->board = reall;
}

void ChessComprBoard::insert(int& pos, unsigned char bits, int n)
{
  assert(n<8);
  assert(bits < (1 << n));

  int 
    bytepos1=pos/8,
    bytepos2=(pos+n)/8,
    startbit=pos%8;
  
  if( bytepos1==bytepos2 )
    compr->board[bytepos1] |= bits << startbit;
  else
    {
      compr->board[bytepos1] |= bits << startbit;
      compr->board[bytepos2] = bits >> (8-startbit);
    }
  
  pos+=n;
}

ChessComprBoard::ChessComprBoard(char * data)
{
  assert(data != 0);
  compr->size = getSize(data);
  fill(compr->board+compr->size,compr->board+sizeof(compr->board), 0);
  copy(data, data+compr->size, compr->board);
}

int ChessComprBoard::getSize(char * data)
{
  return 21;
}

inline
bool isNextBit1(const char * &data, int& bitpos)
{
  bitpos++;
  if( bitpos == 8 )
    {
      bitpos=0;
      data++;
    }
  return (*data & (1 << bitpos)) != 0;
}

    

ChessBoard::ChessBoard(const ChessComprBoard& cb)
{
  int bitpos=-1;
  const char *data=cb.compr->board;
  ChessPiece::piece_t piece;
  for(int index=0; ( index < Chess::XSIZE*Chess::YSIZE ); index++ )
    {
      int count=0;
      while( isNextBit1(data, bitpos) && count < 5 )
	count++;
      
      switch( count )
	{
	case 0:
	  at(index)=ChessPiece(ChessPiece::EMPTY, ChessPiece::WHITE);
	  break;
	case 1:
	  piece=ChessPiece::PAWN;
	  break;
	case 2:
	  if( !isNextBit1(data, bitpos) )
	    piece=ChessPiece::BISHOP;
	  else
	    piece=ChessPiece::KNIGHT;
	  break;
	case 3:
	  piece=ChessPiece::ROOK;
	  break;
	case 4:
	  piece=ChessPiece::QUEEN;
	  break;
	case 5:
	  piece=ChessPiece::KING;
	  bitpos--;
	  break;
	}
      
      if( count > 0 )
	at(index)=ChessPiece(piece, 
			     ChessPiece::color_t(isNextBit1(data, bitpos)));
    }
  int enpassat=0;
  // is this the best solution? think not...
  for( int i=0; i<5; i++)
    {
      enpassat <<=1;
      if( isNextBit1(data, bitpos ) )
	enpassat++;
    }
  if( enpassat != (1 << 4) ) 
    flags.setEnpassat(enpassat);
  flags.setColToMove( ChessPiece::color_t( isNextBit1( data, bitpos ) ) );
  int cast = isNextBit1( data, bitpos ) * ChessBoardFlags::SHORT;
  cast+=isNextBit1(data, bitpos) * ChessBoardFlags::LONG;
  flags.setWCastle( cast );

  cast = isNextBit1( data, bitpos ) * ChessBoardFlags::SHORT;
  cast+=isNextBit1(data, bitpos) * ChessBoardFlags::LONG;
  flags.setBCastle( cast );
}




			  
		  
      
      
      
	
      
      

      
  
    
    
  
  


	   
	  
