/* file: ChessLib/chesscomprboard.h
   author: Christoph Bauer
   date: 22.1.00
   license: GPL

   ChangeLog
     o 22.1.00 created
     
   Description
     Komprimiert eine Stellung mit Hilfe eines Huffmanalgorithmus.
     Dabei ist: (C = Color)
       Empty square: 0
       Pawn: 10C
       Bishop 1100C
       Knight: 1101C
       Rook: 1110C
       Queen: 11110C
       King 11111C

    Performance:
      K6-II/350 MHz braucht 1s um 50000 Anfangsstellungen zu komprimieren und wieder zu 
      dekomprimieren
*/

#ifndef __CHESSCOMPRBOARD_H__
#define __CHESSCOMPRBOARD_H__

#include <iostream>

// we need the "realloc" function
#include <alloc.h>

class ChessBoard;

class ChessComprBoard
{
  friend class ChessBoard;
  struct compr_t
    {
      int refcount;  /* shared */
      size_t size; /* in bytes, for a fast compare */
      char* board; /* for normal 8x8 chessboard!! */
  };
 private:
  compr_t *compr;
  
 public:
  ChessComprBoard(ChessBoard& board);
  ChessComprBoard(const ChessComprBoard& cb)
    : compr(cb.compr)
    { assert(compr != 0); compr->refcount++; }
  ChessComprBoard(char * data);
  ~ChessComprBoard()
    { if( --compr->refcount < 0 ) { free(compr->board); delete compr; } }

  ChessBoard getChessBoard();

  bool operator==(const ChessBoard& board);
  bool operator!=(const ChessBoard& board)
  { return !(this->operator==(board));  }
  
 private:
  void insert(int& pos, unsigned char bits, int n);
  int getSize(char * data);
  
  friend ostream& operator<<(ostream& os, ChessComprBoard cb);
  friend istream& operator>>(istream& is, ChessComprBoard cb);
};

ostream& operator<<(ostream& os, ChessComprBoard cb);
istream& operator>>(istream& is, ChessComprBoard cb);

#endif


