/* file: ChessLib/chessmove.h
   author: Christoph Bauer
   date: 21.1.00
   license: GPL
   
   ChangeLog:
     o 21.1.00 created

   Description
     ChessMove Abstraktion eine Zuges.
*/

#ifndef __CHESSMOVE_H__
#define __CHESSMOVE_H__

#include <string>
#include <iostream>

#include <chesscoord.h>
#include <chesspiece.h>
//#include <chessboard.h>

class ChessBoard;


class ChessMove
{
  ChessCoord start, end;
  ChessPiece::piece_t piece; /* if a pawn reaches the last/first row
	      or diffrent chess rules! */
 public:
  ChessMove(ChessBoard CB, string s); /* s in pgn format */
  ChessMove(string s); /* format like "e2e4" or "a7a8=Q" */
  
  ChessMove(ChessCoord startCoord, ChessCoord endCoord, ChessPiece::piece_t p = ChessPiece::EMPTY )
    : start(startCoord), end(endCoord), piece(p)
    {}
  ChessMove(const ChessMove& m)
    : start(m.start), end(m.end), piece(m.piece)
    {}

  ChessMove(ChessCoord::coord_t x1, ChessCoord::coord_t y1, 
	    ChessCoord::coord_t x2, ChessCoord::coord_t y2, ChessPiece::piece_t p = ChessPiece::EMPTY )
    : start(x1, y1), end(x2, y2), piece(p)
    {}
  
  
  ChessCoord getStart() const
    { return start; }
  ChessCoord getEnd() const
    { return end; }
  ChessPiece::piece_t getPiece() const
    { return piece; }
  friend ostream& operator<<(ostream& os, ChessMove m);
};

extern  
ostream& operator<<(ostream& os, ChessMove m);

  

#endif
