/* file: ChessLib/chesspiece.h
   author: Christoph Bauer
   date: 21.1.00
   license: GPL
   
   ChangeLog
     o 21.1.00 created
     
   Describtion
      Abstraktion von einer Figur.
*/

#ifndef __CHESSPIECE_H__
#define __CHESSPIECE_H__

#include <iostream>

class ChessPiece
{
 public:
  enum piece_t {
    EMPTY, KING, QUEEN, ROOK, BISHOP, KNIGHT, PAWN
  };
  enum color_t {
    WHITE, BLACK, NOCOLOR
  };

  piece_t piece;
  color_t color;
 private:

  static char * pieceCharTab;
  static char* nameTab[];

 public:
  ChessPiece(piece_t which=EMPTY, color_t col=WHITE)
    : piece(which), color(col)
    {}
  ChessPiece(const ChessPiece& c)
    : piece(c.piece), color(c.color)
    {}
  ChessPiece(char c); /* for char like "K" for a white King or "q" for a
			 black Queen */
  char toChar() const;

  const char * toName() const;

  static bool isPieceChar(char c);

  bool empty() const
    { return piece == EMPTY; }

  void setPieceCharTab(char * tab);

  piece_t getPiece() const
    { return piece; }
  color_t getColor() const
    { return color; }

  ChessPiece& operator=(ChessPiece& p)
  { piece=p.piece; color=p.color; return *this; }
  ChessPiece& operator=(const ChessPiece& p)
  { piece=p.piece; color=p.color; return *this; }
  ChessPiece& operator=(char c)
  { return *this=ChessPiece(c); }

  bool operator==(const ChessPiece& p) const
  { return  p.getPiece() == getPiece() && p.getColor() == getColor(); }
  bool operator!=(const ChessPiece& p) const
  { return !operator==(p); }
};

inline 
ostream& operator<<(ostream& os, ChessPiece p)
{ return os << p.toChar(); }
  

  
#endif
