/* file: ChessLib/chesspiececoordtab.h
   author: Christoph Bauer
   date: 6.2.00
   license: GPL
   
   Description:
     Um schneller bei einem Schachbrett auf die Koordinaten zugreifen
     zu koenne, auf denen wirklich auch Figuren stehen, wird in einer
     Tabelle gespeichert, wo welche Figur steht.
*/

#ifndef __CHESSPIECECOORDTAB_H__
#define __CHESSPIECECOORDTAB_H__

#include <chessboard.h>


class ChessPieceCoordTab 
{
  enum { MAXSIZE=16 };
  
  ChessCoord tab[2*MAXSIZE];
  ChessCoord* boardTab[Chess::XSIZE*Chess::YSIZE];
  int size[2];
	 
 public:
  typedef ChessCoord* iterator;
  struct undoMove_t {
    ChessCoord* pos;
    ChessCoord value;
    undoMove_t(ChessCoord* p, ChessCoord val)
      : pos(p), value(val)
    {}
  };
  struct undoErase_t {
    ChessPiece::color_t color;
    ChessCoord c;
    undoErase_t( ChessCoord val,  ChessPiece::color_t col  )
      : c(val), color(col)
    {}
  };
  
  ChessPieceCoordTab( const ChessBoard& b );
  iterator begin(ChessPiece::color_t color) 
    { return tab+color*MAXSIZE; }
  iterator end(ChessPiece::color_t color) 
    { return tab+color*MAXSIZE+size[color]; }
  undoMove_t move( iterator it, ChessCoord& c )
    { undoMove_t undo(it, *it); *it=c; return undo; }
  undoErase_t erase( iterator it );

  void undoMove( const undoMove_t& m )
    { *m.pos=m.value; }
  void undoErase( const undoErase_t& e );

  iterator operator()(ChessCoord i)
    { return boardTab[i.getIndex()]; }
  iterator operator()(int x, int y)
    { return operator()(ChessCoord(x,y)); }
  
    
    
  
    
};


#endif
