#include <chesswxbitmap.h>
#include <algo.h>
#include <wx/dcmemory.h>
#include <fstream>

ChessWxBitmap::ChessWxBitmap(const char *fontname, int pixel, 
			     const ChessFontmap *fontmap )
  : m_font( pixel, wxDEFAULT, wxNORMAL, wxNORMAL, FALSE, fontname ),
    m_pixel( pixel ), m_fontmap(fontmap)
{
  for(int i=0; i < SIZE; i++ )
    m_cache[i]=0;
}

wxBitmap* ChessWxBitmap::getBitmap(const ChessPiece& p )
{
  int index = 
    p.getColor()*(SIZE/2) +
    p.getPiece();
  if( m_cache[index] == 0 ) 
    {
      wxBitmap mask(m_pixel, m_pixel);
      wxMemoryDC dc;
      dc.SelectObject( mask );
      dc.SetBackground( *wxBLACK_BRUSH );
      dc.SetTextForeground( *wxWHITE );
      dc.Clear();
      dc.SetFont(m_font);
      char s[2];
      s[1]=0;
      if( p.getColor() == ChessPiece::WHITE ) 
	{
	  s[0]=m_fontmap->getChar(ChessPiece(p.getPiece(),
					 ChessPiece::WHITE), FALSE);
	  dc.DrawText(s, 0, 0);
	}
      s[0]=m_fontmap->getChar(ChessPiece(p.getPiece(),
					 ChessPiece::BLACK), FALSE);
      dc.DrawText(s, 0, 0);
      wxBitmap *bm = new wxBitmap(m_pixel, m_pixel);
      dc.SelectObject( *bm );
      dc.SetBackground( *wxWHITE_BRUSH );
      dc.Clear();
      dc.SetTextForeground( *wxBLACK );
      s[0]=m_fontmap->getChar(p, FALSE);
      dc.DrawText(s, 0, 0);
      dc.SelectObject(wxNullBitmap);
      bm->SetMask( new wxMask( mask, *wxBLACK ) );
      return m_cache[index]=bm;
    }
  else
    return m_cache[index];
}


void ChessWxBitmap::drawPiece( wxDC& dc, int x, int y, const ChessPiece& p, 
			       bool dark ) 
{
  int index = 
    p.getPiece() + p.getColor() * (SIZE/2);
  wxMemoryDC mdc;
  /*  if( m_cache[index] == 0 )
    {
      wxBitmap *bm = new wxBitmap( m_pixel, m_pixel );
      mdc.SelectObject( *bm );
      mdc.SetTextForeground( *wxWHITE );
      if( p.getColor() == ChessPiece::WHITE )
	mdc.SetTextForeground( *wxWHITE );
      else
	{/ *
	  mdc.SetTextForeground( *wxBLACK );
	  mdc.SetTextBackground( *wxWHITE );
	  mdc.SetBackground( *wxWHITE_BRUSH );
	  mdc.Clear();  * /
	}
      
      char s[2];
      s[1]=0;
      s[0]=toupper(m_fontmap->getChar( ChessPiece( p.getPiece(), ChessPiece::BLACK ) ));
      mdc.SetFont( m_font );
      mdc.DrawText( s, 0, 0 );
      m_cache[index]=bm;
}
  else 
    mdc.SelectObject( *m_cache[index] ); 

  int logic = p.getColor() == ChessPiece::WHITE ? wxOR : wxNAND;
  
  dc.Blit( x, y, m_pixel, m_pixel, &mdc, 0, 0, logic );
  mdc.SelectObject( wxNullBitmap ); */
  
  char s[2];
  s[1]=0;
  s[0]=m_fontmap->getChar( p, dark );
  
  dc.SetFont( m_font );
  dc.DrawText( s, x, y );
}

void ChessWxBitmap::drawBackground( wxDC& dc, int x, int y, bool dark )
{
  char s[2];
  s[1]=0;
  s[0]=m_fontmap->getBackgroundChar( dark );
  dc.DrawText( s, x, y );
}

ChessWxBitmap::~ChessWxBitmap()
{
  for( int i=0; i < SIZE; i++ )
    delete m_cache[i];
}

ChessFontmap::ChessFontmap( const char * data )
  : toFree( 0 )
{
  assert( data == 0 || strlen(data) >= SIZE );
  table=data;
}

void ChessFontmap::load(char * filename)
{
  ifstream in(filename);
  free(toFree);
  char s[SIZE];
  in.get(s,SIZE);
  assert( strlen(s) == SIZE );
  table=toFree=strdup(s);
}

ChessFontmap::~ChessFontmap()
{
  delete toFree;
}

char ChessFontmap::getChar( const ChessPiece& p, bool dark  ) const
{
  return table[p.getPiece()+p.getColor()*(SIZE/4)+dark*(SIZE/2)];
}

char ChessFontmap::getBackgroundChar( bool dark ) const
{
  return dark ? ' ' : '/';
}






    


