/* file: ChessLib/chesswxbitmap.h
   author: Christoph Bauer
   date: 13.02.00
   license: GPL
   
   Description:
     Liefert fuer wxWindows zu einer passenden Figur ein Bitmap
     Auch Transparente.

     Gemacht wird das mit TrueType fonts.
     -> fuer X braucht man entweder XFree86 4.0 (3.9.17 funktioniert bei mir)
     oder einen TrueType font server (die waren bei mir sehr instabil.
     wenn z.B. xtstt abstuerzt dann haengt der X-Server (was auf einem 
     Bug im X-Server zurueckzufuehren ist.)

     Die Fonts gibt es bei 
       http://www.chessvariants.com/d.font/index.html
*/
#ifndef __CHESSWXBITMAP_H__
#define __CHESSWXBITMAP_H__

#include <wx/font.h>
#include <wx/bitmap.h>
#include <chesspiece.h>

class wxDC;

class ChessFontmap
{
  const char *table;
  char *toFree;
 public:
  enum { SIZE=(ChessPiece::PAWN+1)*4 };
  ChessFontmap(const char * data = 0);
  virtual ~ChessFontmap();
  void load(char * filename);

  virtual char getChar( const ChessPiece& p, bool dark ) const;
  virtual char getBackgroundChar( bool dark ) const;
};

class ChessWxBitmap
{
 public:
  enum { SIZE=(ChessPiece::PAWN+1)*2 };
 private:
  wxFont m_font;
  int m_pixel;
  wxBitmap *m_cache[SIZE];
  const ChessFontmap * m_fontmap;
  
 public:
  ChessWxBitmap(const char * fontname, int pixel, 
		const ChessFontmap *fontmap );
  ~ChessWxBitmap();

  wxBitmap* getBitmap( const ChessPiece& p );
  void drawPiece( wxDC& dc, int x, int y, const ChessPiece& p, bool dark );
  int getPixel()
    { return m_pixel; }
  void drawBackground( wxDC& dc, int x, int y, bool dark );
  //  void draw
};

#endif


  
  
