/* file: parser/parser.h
   author: Christoph Bauer
   date: 5.2.00
   license: GPL

   Description:
     Erweiterung fuer re2c zur einfachen Benutzung
*/

#ifndef __PARSER_H__
#define __PARSER_H__

#include <stdio.h>
#include <string>

template<class yyval_t>
class Parser
{
 public:
  typedef unsigned char uchar ;
  typedef unsigned int uint;
  
 protected:
  int line;
  int word;
  
  int in;
  yyval_t  yyval;
  uchar *cursor, *lim, *ptr, *eof, *tok, *bot, *pos, *top;
  size_t bsize;

 public:
  Parser(int fd, size_t blocksize = 8192);
  ~Parser();

  virtual void init()
    { initParser(); }
  void done()
    { doneParser(); }
  
  virtual void run()=0;
  int getLine()
    { return line; }
  int getWord()
    { return word; }

  string getToken()
    { string res((char*)tok, cursor-tok); 
      tok=cursor; return res;  }
  
 protected:
  uchar * fill(uchar * c, uint n);
  void nextLine()
    { line++; }
  void nextWord()
    { word++; }
  virtual void callParser(int id) {};
  virtual void initParser() {};
  virtual void doneParser() {};
  virtual void fatal();

};

#define YYCTYPE uchar
#define YYCURSOR cursor
#define YYLIMIT lim
#define YYMARKER ptr
#define YYFILL(n) { cursor = fill(cursor,n); } 

#include <parser.cpp>
	 
#endif
