#include <wx/bitmap.h>
#include <wx/font.h>
#include <chessboard.h>
#include <wx/dcmemory.h>
#include <wxboard.h>
#include <chesswxbitmap.h>

wxBoard::wxBoard( ChessWxBitmap* piece,
		  const ChessBoard& board,
		  wxFrame * parent, const wxString& title)
  : wxFrame( parent, -1, title, wxDefaultPosition, 
	     wxSize( 10 + piece->getPixel()*Chess::XSIZE, 10 + piece->getPixel()*Chess::YSIZE)),
    m_isDown(FALSE), m_save(0),
    m_x(-1), m_y(-1),
    m_p(ChessPiece::EMPTY, ChessPiece::WHITE ),
    m_board( new ChessBoard( board ) ),  
    m_piece(piece),
    m_xborder(5), m_yborder(5)
  
{
  /*  m_field[0] = new wxBitmap( pixel, pixel );
  m_field[1] = new wxBitmap( pixel, pixel );
  wxMemoryDC dc;
  dc.SelectObject( *m_field[0] );
  dc.SetFont( *m_font );
  dc.DrawText( m_fontmap->getBackgroundChar( 0 ), 0, 0 );
  dc.SelectObject( *m_field[1] );
  dc.DrawText( m_fontmap->getBackgroundChar( 1 ), 0, 0 ); */
}

wxBoard::~wxBoard()
{
  if( m_save )
    delete m_save;
  delete m_font;
  delete m_board;
}


void wxBoard::setBoard( const ChessBoard& board )
{
  wxClientDC dc(this);
  draw( dc, board );
  *m_board = board;
}

void wxBoard::OnPaint( wxPaintEvent& )
{
  wxPaintDC dc(this);
  dc.BeginDrawing();
  draw( dc, *m_board, TRUE );
  dc.EndDrawing();
}


int wxBoard::GetXPixel( ChessCoord::coord_t x ) const
{
  return m_xborder+m_piece->getPixel()*x;
}


int wxBoard::GetYPixel( ChessCoord::coord_t y ) const
{
  return m_yborder+m_piece->getPixel()*y;
}

void wxBoard::updateField( wxDC& dc,
			   ChessCoord::coord_t x, ChessCoord::coord_t y,
			   const ChessPiece& p )
{
  int 
    xpos=GetXPixel( x ),
    ypos=GetYPixel( y );
  m_piece->drawBackground( dc, xpos, ypos, x%2 ^ y%2 );
  if( !p.empty() )
    m_piece->drawPiece( dc, xpos, ypos, p, x%2 ^ y%2 );
}  

void wxBoard::draw( wxDC& dc, const ChessBoard& newBoard, bool dirty )
{
  for( int y=0; y < Chess::YSIZE; y++ )
    for( int x=0; x < Chess::XSIZE; x++ )
      if( dirty ||
	  newBoard.at( x, y ) != m_board->at( x, y ) )
	updateField( dc, x, 7-y, newBoard.at( x, y ) );
}





BEGIN_EVENT_TABLE(wxBoard, wxFrame)
  EVT_PAINT(wxBoard::OnPaint)
END_EVENT_TABLE()

  






