/* file: ChessLib/wxboard.h
   author: Christoph Bauer
   date: 13.2.00
   license: GPL

   Description:
     Schachbrett mit wxWindows
*/

#ifndef __WXBOARD_H__
#define __WXBOARD_H__

#include <chessconfig.h>

#ifdef WITHWXWINDOWS

#include <chesspiece.h>
#include <wx/frame.h>
#include <chessboard.h>

class wxBitmap;
class wxFont;
class ChessBoard;
class ChessWxBitmap;
class ChessMove;


class wxBoard : public wxFrame
{
  bool m_isDown;
  wxBitmap* m_save;
  int m_x, m_y;
  wxFont *m_font;
  ChessPiece m_p;
  // wxBitmap* m_field[2];
  ChessBoard* m_board;
  ChessWxBitmap* m_piece;
  int m_xborder, m_yborder;

 public:
  wxBoard( ChessWxBitmap* piece, 
	   const ChessBoard& board, 
	   wxFrame * parent=0, const wxString& title="");
  ~wxBoard();
  
  void setBoard( const ChessBoard& board );
  void move( const ChessMove& move );

  void OnPaint( wxPaintEvent& );
  void OnLeftDown( wxMouseEvent& e );
  void OnLeftUp( wxMouseEvent& e );
  void OnMotion( wxMouseEvent& e );

  void draw(wxDC& dc, const ChessBoard& newBoard, bool dirty = FALSE );
  
  void updateField( wxDC& dc,
		    ChessCoord::coord_t x, ChessCoord::coord_t y,
		    const ChessPiece& p);

  int GetXPixel( ChessCoord::coord_t x ) const;
  int GetYPixel( ChessCoord::coord_t y ) const; 

  DECLARE_EVENT_TABLE()
};


#endif
#endif
